@@ -10,7 +10,7 @@ bb_wrap = function(bb) {
 }
 
 bbox.Set = function(obj, ...) {
-	sel = vapply(obj, function(x) { length(x) && !all(is.na(x)) }, TRUE)
+	sel = !sfc_is_empty(obj)
 	if (! any(sel))
 		NA_bbox_
 	else
@@ -130,7 +130,7 @@ st_bbox.CIRCULARSTRING = function(obj, ...) {
 #' @export
 print.bbox = function(x, ...) {
 	x = structure(x, crs = NULL, class = NULL)
-	print(set_units(x, attr(x, "units"), mode = "standard"))
+	print(set_units(x, attr(x, "units"), mode = "standard"), ...)
 }
 
 compute_bbox = function(obj) {
