@@ -20,7 +20,7 @@
 #' @param lty line type
 #' @param lwd line width
 #' @param col color for plotting features; if \code{length(col)} does not equal 1 or \code{nrow(x)}, a warning is emitted that colors will be recycled. Specifying \code{col} suppresses plotting the legend key.
-#' @param border color of polygon border(s)
+#' @param border color of polygon border(s); using \code{NA} hides them
 #' @param add logical; add to current plot? Note that when using \code{add=TRUE}, you may have to set \code{reset=FALSE} in the first plot command.
 #' @param type plot type: 'p' for points, 'l' for lines, 'b' for both
 #' @param reset logical; if \code{FALSE}, keep the plot in a mode that allows adding further map elements; if \code{TRUE} restore original mode after plotting \code{sf} objects with attributes; see details.
@@ -64,7 +64,7 @@
 #' .image_scale(1:10, col = sf.colors(9), key.length = lcm(8), key.pos = 4, at = 1:10)
 #' @export
 plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty",
-		max.plot = if(is.null(n <- options("sf_max.plot")[[1]])) 9 else n,
+		max.plot = if(is.null(n <- getOption("sf_max.plot"))) 9 else n,
 		key.pos = get_key_pos(x, ...), key.length = .618, key.width = lcm(1.8),
 		reset = TRUE, logz = FALSE, extent = x, xlim = st_bbox(extent)[c(1,3)],
 		ylim = st_bbox(extent)[c(2,4)]) {
@@ -118,6 +118,8 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 				nbreaks = length(breaks) - 1
 			}
 		}
+		if (missing(nbreaks) && is.numeric(breaks))
+			nbreaks = length(breaks) - 1
 
 		# loop over each map to plot:
 		lapply(cols, function(cname) plot(x[, cname], main = cname,
@@ -185,10 +187,10 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 							v0 = values[!is.na(values)]
 							n.unq = length(unique(v0))
 							breaks = if (! all(is.na(values)) && n.unq > 1)
-								classInt::classIntervals(v0, min(nbreaks, n.unq),
-									breaks, warnSmallN = FALSE)$brks
-							else
-								range(values, na.rm = TRUE) # lowest and highest!
+									classInt::classIntervals(v0, min(nbreaks, n.unq),
+										breaks, warnSmallN = FALSE)$brks
+								else
+									range(values, na.rm = TRUE) # lowest and highest!
 						}
 						# this is necessary if breaks were specified either as character or as numeric
 						# "pretty" takes nbreaks as advice only:
@@ -236,10 +238,12 @@ plot.sf <- function(x, y, ..., main, pal = NULL, nbreaks = 10, breaks = "pretty"
 						key.length = key.length, logz = logz, ...)
 			}
 			# plot the map:
-			mar = c(1, 1, 1.2, 1)
-			if (isTRUE(dots$axes))
-				mar[1:2] = 2.1
-			par(mar = mar)
+			if (!isTRUE(dots$add)) {
+				mar = c(1, 1, 1.2, 1)
+				if (isTRUE(dots$axes))
+					mar[1:2] = 2.1
+				par(mar = mar)
+			}
 			if (col_missing)
 				plot(st_geometry(x), col = col, xlim = xlim, ylim = ylim, ...)
 			else
@@ -401,10 +405,11 @@ p_bind = function(lst) {
 
 #' @name plot
 #' @param rule see \link[graphics]{polypath}; for \code{winding}, exterior ring direction should be opposite that of the holes; with \code{evenodd}, plotting is robust against misspecified ring directions
+#' @param xpd see \link[graphics]{par}; sets polygon clipping strategy; only implemented for POLYGON and MULTIPOLYGON
 #' @export
 plot.sfc_POLYGON = function(x, y, ..., lty = 1, lwd = 1, col = NA, cex = 1, pch = NA, border = 1,
-		add = FALSE, rule = "evenodd") {
-# FIXME: take care of lend, ljoin, xpd, and lmitre
+		add = FALSE, rule = "evenodd", xpd = par("xpd")) {
+# FIXME: take care of lend, ljoin, and lmitre
 	stopifnot(missing(y))
 	if (! add)
 		plot_sf(x, ...)
@@ -415,7 +420,7 @@ plot.sfc_POLYGON = function(x, y, ..., lty = 1, lwd = 1, col = NA, cex = 1, pch
 	non_empty = ! st_is_empty(x)
 	lapply(seq_along(x), function(i)
 	  if (non_empty[i])
-		polypath(p_bind(x[[i]]), border = border[i], lty = lty[i], lwd = lwd[i], col = col[i], rule = rule))
+		polypath(p_bind(x[[i]]), border = border[i], lty = lty[i], lwd = lwd[i], col = col[i], rule = rule, xpd = xpd))
 #	if (any(!is.na(pch))) {
 #		pch = rep(pch, length.out = length(x))
 #		cex = rep(cex, length.out = length(x))
@@ -429,8 +434,9 @@ plot.sfc_POLYGON = function(x, y, ..., lty = 1, lwd = 1, col = NA, cex = 1, pch
 #' @name plot
 #' @method plot sfc_MULTIPOLYGON
 #' @export
-plot.sfc_MULTIPOLYGON = function(x, y, ..., lty = 1, lwd = 1, col = NA, border = 1, add = FALSE, rule = "evenodd") {
-# FIXME: take care of lend, ljoin, xpd, and lmitre
+plot.sfc_MULTIPOLYGON = function(x, y, ..., lty = 1, lwd = 1, col = NA, border = 1, add = FALSE, 
+		rule = "evenodd", xpd = par("xpd")) {
+# FIXME: take care of lend, ljoin, and lmitre
 	stopifnot(missing(y))
 	if (! add)
 		plot_sf(x, ...)
@@ -442,7 +448,7 @@ plot.sfc_MULTIPOLYGON = function(x, y, ..., lty = 1, lwd = 1, col = NA, border =
 	lapply(seq_along(x), function(i)
 	  if (non_empty[i])
 		lapply(x[[i]], function(L)
-			polypath(p_bind(L), border = border[i], lty = lty[i], lwd = lwd[i], col = col[i], rule = rule)))
+			polypath(p_bind(L), border = border[i], lty = lty[i], lwd = lwd[i], col = col[i], rule = rule, xpd = xpd)))
 	invisible(NULL)
 }
 
@@ -549,14 +555,14 @@ plot.sfg = function(x, ...) {
 #'
 plot_sf = function(x, xlim = NULL, ylim = NULL, asp = NA, axes = FALSE, bgc = par("bg"), ...,
     xaxs, yaxs, lab, setParUsrBB = FALSE, bgMap = NULL, expandBB = c(0,0,0,0), graticule = NA_crs_,
-	col_graticule = 'grey', border) {
+	col_graticule = 'grey', border, extent = x) {
 
 # sp's bbox: matrix
 #   min max
 # x
 # y
 
-	bbox = matrix(st_bbox(x), 2, dimnames = list(c("x", "y"), c("min", "max")))
+	bbox = matrix(st_bbox(extent), 2, dimnames = list(c("x", "y"), c("min", "max")))
 	# expandBB: 1=below, 2=left, 3=above and 4=right.
 	expBB = function(lim, expand) c(lim[1] - expand[1] * diff(lim), lim[2] + expand[2] * diff(lim))
 	if (is.null(xlim))
@@ -672,6 +678,8 @@ sf.colors = function (n = 10, cutoff.tails = c(0.35, 0.2), alpha = 1, categorica
 # return list with "m" matrix, "key.pos", "widths" and "heights" fields
 # if key.pos = -1, it will be a return value, "optimally" placed
 	asp = diff(bb[c(2,4)])/diff(bb[c(1,3)])
+	if (!is.finite(asp)) # 0/0
+		asp = 1
 	if (isTRUE(st_is_longlat(bb)))
 		asp = asp / cos(mean(bb[c(2,4)]) * pi /180)
 	if (is.null(mfrow)) {
