@@ -121,7 +121,7 @@ set_utf8 = function(x) {
 #' # spatial filter, as wkt:
 #' wkt = st_as_text(st_geometry(nc[1,]))
 #' # filter by (bbox overlaps of) first feature geometry:
-#' read_sf(system.file("gpkg/nc.gpkg", package="sf"), wkt_filter = wkt)
+#' st_read(system.file("gpkg/nc.gpkg", package="sf"), wkt_filter = wkt)
 #' @export
 st_read = function(dsn, layer, ...) UseMethod("st_read")
 
@@ -142,7 +142,7 @@ st_read.default = function(dsn, layer, ...) {
 
 process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 		stringsAsFactors = ifelse(as_tibble, FALSE, sf_stringsAsFactors()), 
-		geometry_column = 1, as_tibble = FALSE) {
+		geometry_column = 1, as_tibble = FALSE, optional = FALSE) {
 
 	which.geom = which(vapply(x, function(f) inherits(f, "sfc"), TRUE))
 
@@ -155,8 +155,8 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 			warning("no simple feature geometries present: returning a data.frame or tbl_df", call. = FALSE)
 		x = if (!as_tibble) {
 				if (any(sapply(x, is.list)))
-					warning("list-column(s) present: in case of failure, try read_sf or as_tibble=TRUE") # nocov
-				as.data.frame(x , stringsAsFactors = stringsAsFactors)
+					warning("list-column(s) present: in case of failure, try read_sf() or as_tibble=TRUE") # nocov
+				as.data.frame(x , stringsAsFactors = stringsAsFactors, optional = optional)
 			} else
 				tibble::as_tibble(x)
 		return(x)
@@ -176,7 +176,8 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 		else
 			x <- data.frame(row.names = seq_along(geom[[1]]))
 	} else {
-		x <- as.data.frame(set_utf8(x[-c(lc.other, which.geom)]), stringsAsFactors = stringsAsFactors)
+		x <- as.data.frame(set_utf8(x[-c(lc.other, which.geom)]), stringsAsFactors = stringsAsFactors,
+			optional = optional || as_tibble)
 		if (as_tibble) {
 			# "sf" class is added later by `st_as_sf` (and sets all the attributes)
 			x <- tibble::new_tibble(x, nrow = nrow(x))
@@ -202,6 +203,7 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 #' @param fid_column_name character; name of column to write feature IDs to; defaults to not doing this
 #' @param drivers character; limited set of driver short names to be tried (default: try all)
 #' @param wkt_filter character; WKT representation of a spatial filter (may be used as bounding box, selecting overlapping geometries); see examples
+#' @param optional logical; passed to \link[base]{as.data.frame}; always \code{TRUE} when \code{as_tibble} is \code{TRUE}
 #' @note The use of \code{system.file} in examples make sure that examples run regardless where R is installed:
 #' typical users will not use \code{system.file} but give the file name directly, either with full path or relative
 #' to the current working directory (see \link{getwd}). "Shapefiles" consist of several files with the same basename
@@ -210,15 +212,14 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet = FALSE, geometry_column = 1L, 
 		type = 0, promote_to_multi = TRUE, stringsAsFactors = sf_stringsAsFactors(),
 		int64_as_string = FALSE, check_ring_dir = FALSE, fid_column_name = character(0),
-		drivers = character(0), wkt_filter = character(0)) {
+		drivers = character(0), wkt_filter = character(0), optional = FALSE) {
 
 	layer = if (missing(layer))
 		character(0)
 	else
 		enc2utf8(layer)
-	if (nchar(dsn) < 1) {
+	if (nchar(dsn) < 1)
 		stop("`dsn` must point to a source, not an empty string.", call. = FALSE)
-	}
 	dsn_exists = file.exists(dsn)
 	dsn_isdb = is_db_driver(dsn)
 	if (length(dsn) == 1 && dsn_exists && !dsn_isdb)
@@ -230,7 +231,8 @@ st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet
 	x = CPL_read_ogr(dsn, layer, query, as.character(options), quiet, type, fid_column_name,
 		drivers, wkt_filter, promote_to_multi, int64_as_string, dsn_exists, dsn_isdb, getOption("width"))
 	process_cpl_read_ogr(x, quiet, check_ring_dir = check_ring_dir,
-		stringsAsFactors = stringsAsFactors, geometry_column = geometry_column, ...)
+		stringsAsFactors = stringsAsFactors, geometry_column = geometry_column, 
+		optional = optional, ...)
 }
 
 #' @name st_read
@@ -244,7 +246,7 @@ st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet
 #' # read geojson from string:
 #' geojson_txt <- paste("{\"type\":\"MultiPoint\",\"coordinates\":",
 #'    "[[3.2,4],[3,4.6],[3.8,4.4],[3.5,3.8],[3.4,3.6],[3.9,4.5]]}")
-#' x = read_sf(geojson_txt)
+#' x = st_read(geojson_txt)
 #' x
 read_sf <- function(..., quiet = TRUE, stringsAsFactors = FALSE, as_tibble = TRUE) {
 	st_read(..., quiet = quiet, stringsAsFactors = stringsAsFactors, as_tibble = as_tibble)
@@ -367,12 +369,13 @@ abbreviate_shapefile_names = function(x) {
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' st_write(nc, paste0(tempdir(), "/", "nc.shp"))
 #' st_write(nc, paste0(tempdir(), "/", "nc.shp"), delete_layer = TRUE) # overwrites
-#' data(meuse, package = "sp") # loads data.frame from sp
-#' meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992)
-#' # writes X and Y as columns:
-#' st_write(meuse_sf, paste0(tempdir(), "/", "meuse.csv"), layer_options = "GEOMETRY=AS_XY")
-#' st_write(meuse_sf, paste0(tempdir(), "/", "meuse.csv"), layer_options = "GEOMETRY=AS_WKT",
-#'   delete_dsn=TRUE) # overwrites
+#' if (require(sp, quietly = TRUE)) {
+#'  data(meuse, package = "sp") # loads data.frame from sp
+#'  meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992)
+#'  # writes X and Y as columns:
+#'  st_write(meuse_sf, paste0(tempdir(), "/", "meuse.csv"), layer_options = "GEOMETRY=AS_XY")
+#'  st_write(meuse_sf, paste0(tempdir(), "/", "meuse.csv"), layer_options = "GEOMETRY=AS_WKT",
+#'    delete_dsn=TRUE) # overwrites
 #' \dontrun{
 #'  library(sp)
 #'  example(meuse, ask = FALSE, echo = FALSE)
@@ -381,6 +384,7 @@ abbreviate_shapefile_names = function(x) {
 #'  demo(nc, ask = FALSE)
 #'  try(st_write(nc, "PG:dbname=postgis", "sids", layer_options = "OVERWRITE=true"))
 #' }
+#' }
 #' @export
 st_write = function(obj, dsn, layer, ...) UseMethod("st_write")
 
@@ -566,25 +570,35 @@ print.sf_layers = function(x, ...) {
 		cat("<none>\n") # nocov
 		invisible(x)    # nocov
 	} else {
+		crs = sapply(x$crs, function(crs) crs$input)
+		x$crs = crs
 		df = data.frame(unclass(x))
 		gt = if (n_gt > 1)
 				"geometry_types"
 			else
 				"geometry_type"
-		names(df) = c("layer_name", gt, "features", "fields")
+		names(df) = c("layer_name", gt, "features", "fields", "crs_name")
 		print(df)
 		invisible(df)
 	}
 }
 
-#' List layers in a datasource
+#' Return properties of layers in a datasource
 #'
-#' List layers in a datasource
+#' Return properties of layers in a datasource
 #' @param dsn data source name (interpretation varies by driver - for some drivers, \code{dsn} is a file name, but may also be a
 #' folder, or contain the name and access credentials of a database)
 #' @param options character; driver dependent dataset open options, multiple options supported.
 #' @param do_count logical; if TRUE, count the features by reading them, even if their count is not reported by the driver
 #' @name st_layers
+#' @return list object of class \code{sf_layers} with elements 
+#' \describe{
+#'   \item{name}{name of the layer}
+#'   \item{geomtype}{list with for each layer the geometry types}
+#'   \item{features}{number of features (if reported; see \code{do_count})}
+#'   \item{fields}{number of fields}
+#'   \item{crs}{list with for each layer the \code{crs} object}
+#' }
 #' @export
 st_layers = function(dsn, options = character(0), do_count = FALSE) {
 	if (missing(dsn))
@@ -678,6 +692,7 @@ extension_map <- list(
         "gmt" = "GMT",
         "gpkg" = "GPKG",
         "gps" = "GPSBabel",
+        "gpx" = "GPX",
         "gtm" = "GPSTrackMaker",
         "gxt" = "Geoconcept",
         "jml" = "JML",
