@@ -16,7 +16,7 @@ st_as_sf = function(x, ...) UseMethod("st_as_sf")
 #' @param sf_column_name character; name of the active list-column with simple feature geometries; in case
 #' there is more than one and \code{sf_column_name} is \code{NULL}, the first one is taken.
 #' @param ... passed on to \link{st_sf}, might included named arguments \code{crs} or \code{precision}
-#' @details setting argument \code{wkt} annihilates the use of argument \code{coords}. If \code{x} contains a column called "geometry", \code{coords} will result in overwriting of this column by the \link{sfc} geometry list-column.  Setting \code{wkt} will replace this column with the geometry list-column, unless \code{remove_coordinates} is \code{FALSE}.
+#' @details setting argument \code{wkt} annihilates the use of argument \code{coords}. If \code{x} contains a column called "geometry", \code{coords} will result in overwriting of this column by the \link{sfc} geometry list-column.  Setting \code{wkt} will replace this column with the geometry list-column, unless \code{remove} is \code{FALSE}.
 #'
 #' @examples
 #' pt1 = st_point(c(0,1))
@@ -29,10 +29,12 @@ st_as_sf = function(x, ...) UseMethod("st_as_sf")
 #' df = st_as_sf(d, wkt = "geom")
 #' d$geom2 = st_sfc(pt1, pt2)
 #' st_as_sf(d) # should warn
-#' data(meuse, package = "sp")
-#' meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
-#' meuse_sf[1:3,]
-#' summary(meuse_sf)
+#' if (require(sp, quietly = TRUE)) {
+#'  data(meuse, package = "sp")
+#'  meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
+#'  meuse_sf[1:3,]
+#'  summary(meuse_sf)
+#' }
 #' @export
 st_as_sf.data.frame = function(x, ..., agr = NA_agr_, coords, wkt,
 		dim = "XYZ", remove = TRUE, na.fail = TRUE, sf_column_name = NULL) {
@@ -450,13 +452,21 @@ as.data.frame.sf = function(x, ...) {
 
 #' @export
 #' @name st_geometry
-#' @details \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly
-st_drop_geometry = function(x) {
-	if (!inherits(x, "sf"))
-		stop("st_drop_geometry only works with objects of class sf")
+#' @details if \code{x} is of class \code{sf}, \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly; otherwise it returns \code{x} unmodified. 
+st_drop_geometry = function(x, ...) UseMethod("st_drop_geometry")
+
+#' @export
+#' @name st_geometry
+st_drop_geometry.sf = function(x, ...) {
 	st_set_geometry(x, NULL)
 }
 
+#' @export
+#' @name st_geometry
+st_drop_geometry.default = function(x, ...) {
+	x
+}
+
 #' transform method for sf objects
 #' 
 #' Can be used to create or modify attribute variables; for transforming geometries see 
