@@ -3,7 +3,7 @@
 #' @param x character vector, possibly of length larger than 1 when more than one raster is read
 #' @param ... ignored
 #' @param options character; raster layer read options
-#' @param driver character; when empty vector, driver is auto-detected.
+#' @param driver character; driver short name; when empty vector, driver is auto-detected.
 #' @param read_data logical; if \code{FALSE}, only the imagery metadata is returned
 #' @param NA_value (double) non-NA value to use for missing values; if \code{NA}, when writing missing values are not specially flagged in output dataset, when reading the default (dataset) missing values are used (if present / set).
 #' @param RasterIO_parameters list with named parameters to GDAL's RasterIO; see the stars::read_stars documentation.
@@ -43,22 +43,42 @@ gdal_write = function(x, ..., file, driver = "GTiff", options = character(0), ty
 		from = c(0, 0) # nocov end
 	} else {
 		mat = x[[1]]
+		dm = dim(mat)
+		if (is.factor(mat)) {
+			rgba = NULL
+			ex = attr(mat, "exclude")
+			if (is.null(ex))
+				lev = c("", levels(mat)) # add "" for value 0: R factors start at 1
+			else {
+				if (any(ex)) {
+					lev = vector("character", length(ex)) # fills with ""
+					lev[!ex] = levels(mat)
+					rgba = if (!is.null(co <- attr(mat, "rgba"))) {
+						n = length(ex)
+						coltab = cbind(rep(0., n), rep(0, n), rep(0, n), rep(255, n))
+						coltab[!ex,] = co
+						coltab 
+					}
+					values = which(!ex) - 1
+					mat = values[as.numeric(mat)]
+				} else
+					lev = levels(mat)
+			}
+			mat = structure(mat, class = NULL, levels = lev, dim = dm, rgba = rgba)
+		}
 		only_create = FALSE # write x too
 		if (! update) {
 			if (!all(from == 0))
 				stop("cannot write sub-rasters only")
-			if (!all(dims == dim(mat)))
+			if (!all(dims == dm))
 				stop("dimensions don't match")
 		}
-		dm = dim(mat)
 		dim(mat) = c(dm[1], prod(dm[-1])) # flatten to 2-D matrix
 	}
 	if (length(dims) == 2)
 		dims = c(dims, 1) # one band
-	else { # add band descriptions?
-		if (is.character(d[[3]]$values))
-			attr(mat, "descriptions") = d[[3]]$values
-	}
+	else if (is.character(d[[3]]$values)) # add band descriptions?
+		attr(mat, "descriptions") = d[[3]]$values
 
 	CPL_write_gdal(mat, file, driver, options, type, dims, from, geotransform,
 		st_crs(x)[[2]], as.double(NA_value), create = !update, only_create = only_create)
@@ -121,13 +141,15 @@ st_as_sfc.dimensions = function(x, ..., as_points = NA, use_cpp = TRUE, which =
 	yd = x[[ xy_names[2] ]]
 	cc = if (!is.na(xd$offset) && !is.na(yd$offset)) {
 		xy = if (as_points) # grid cell centres:
-			expand.grid(x = seq(xd$from, xd$to) - 0.5, y = seq(yd$from, yd$to) - 0.5)
-		else # grid corners: from 0 to n
-			expand.grid(x = seq(xd$from - 1, xd$to), y = seq(yd$from - 1, yd$to))
+				expand.grid(x = seq(xd$from, xd$to) - 0.5, y = seq(yd$from, yd$to) - 0.5)
+			else # grid corners: from 0 to n
+				expand.grid(x = seq(xd$from - 1, xd$to), y = seq(yd$from - 1, yd$to))
 		xy_from_colrow(as.matrix(xy), geotransform)
 	} else if (is.null(xd$values) || is.null(yd$values)) { # only one of [xd|yd] has $values:
 		if (!requireNamespace("stars", quietly = TRUE)) # nocov start
 			stop("stars required: install that first")
+		if (! as_points)
+			stop("st_as_sfc(): mixed regular and rectilinear dimensions only supported if as_points = TRUE")
 		as.matrix(st_coordinates(x)) # nocov end
 	} else { # both xd and yd have $values:
 		expand = function(x) { # might fail on the poles or dateline
@@ -210,8 +232,7 @@ gdal_metadata = function(file, domain_item = character(0), options = character(0
 split_strings = function(md, split = "=") {
 	splt = strsplit(md, split)
 	lst = lapply(splt, function(x) if (length(x) <= 1) NA_character_ else x[[2]])
-	structure(lst, names = sapply(splt, function(x) x[[1]]))
-	structure(lst, class = "gdal_metadata")
+	structure(lst, names = sapply(splt, function(x) x[[1]]), class = "gdal_metadata")
 }
 
 #' @param name logical; retrieve name of subdataset? If \code{FALSE}, retrieve description
@@ -256,7 +277,7 @@ gdal_polygonize = function(x, mask = NULL, file = tempfile(), driver = "GTiff",
 			if (max(breaks) == max(x[[1]], na.rm = TRUE)) # expand, because GDAL will not include interval RHS
 				nbreaks[length(nbreaks)] = breaks[length(breaks)] * 1.01
 			c(paste0("FIXED_LEVELS=", paste0(nbreaks, collapse = ",")),
-			paste0("ELEV_FIELD=Value"),
+			"ELEV_FIELD=0", "ELEV_FIELD_MIN=1", "ELEV_FIELD_MAX=2",
 			paste0("POLYGONIZE=", ifelse(contour_lines, "NO", "YES")))
 		} else
 			character(0)
@@ -266,16 +287,27 @@ gdal_polygonize = function(x, mask = NULL, file = tempfile(), driver = "GTiff",
 		else
 			character(0)
 
-	#pol = CPL_polygonize(file, mask_name, "GTiff", "GPKG", "foo", options, 0, contour_options, use_contours, use_integer)
-	pol = CPL_polygonize(file, mask_name, "GTiff", "Memory", "foo", options, 0, contour_options, use_contours, use_integer)
+	pol = CPL_polygonize(file, mask_name, "GTiff", "Memory", "foo", options, 0, 
+		contour_options, use_contours, use_integer)
 	out = process_cpl_read_ogr(pol, quiet = TRUE)
 	names(out)[1] = names(x)[1]
-	if (use_contours) {
-		m = as.integer(cut(out[[1]], breaks = nbreaks, include.lowest = TRUE)) # FIXME: add coverage when GDAL 2.4.0 is here
-		if (any(is.na(m)) && !all(is.na(m)))
-			warning("range of breaks does not cover range of cell values")
-		out[[1]] = structure(m, levels = levels(cut(breaks, breaks, include.lowest = TRUE)), class = "factor")
-	}
+	if (! contour_lines && use_contours) {
+#		if (out$Min[1] == 0 && out$Min[1] > min(breaks))
+#			out$Min[1] = -Inf
+#
+# https://github.com/r-spatial/sf/pull/1608 : 
+
+		i <- match(out$Max[1], sort(breaks))
+		out$Min[1] = if (!is.na(i) && i > 1)
+				sort(breaks)[i - 1]
+			else 
+				-Inf
+
+		out$Max[out$Max == 2^32 - 1] = Inf
+		f = paste0("[", out$Min, ",", out$Max, ")")
+		out[[1]] = factor(f, levels = f)
+	} else
+		out$Min = out$Max = NULL
 	out # nocov end
 }
 
@@ -284,7 +316,9 @@ gdal_polygonize = function(x, mask = NULL, file = tempfile(), driver = "GTiff",
 #' @export
 gdal_rasterize = function(sf, x, gt, file, driver = "GTiff", options = character()) {
 	gdal_write(x, file = file, driver = driver, geotransform = gt)
-	CPL_rasterize(file, driver, st_geometry(sf), as.double(as.data.frame(sf)[[1]]), options, NA_real_);
+	geoms = which(sapply(sf, inherits, "sfc"))
+	values = as.double(t(as.matrix(as.data.frame(sf)[-geoms])))
+	CPL_rasterize(file, driver, st_geometry(sf), values, options, NA_real_);
 }
 
 #' @export
@@ -295,3 +329,66 @@ gdal_rasterize = function(sf, x, gt, file, driver = "GTiff", options = character
 gdal_extract = function(f, pts, bilinear = FALSE) {
 	CPL_extract(f, pts, as.logical(bilinear))
 }
+
+#' @name gdal
+#' @param file file name
+#' @param array_name array name
+#' @param offset offset (pixels)
+#' @param count number of pixels to read
+#' @param step step size (pixels)
+#' @param proxy logical; return proxy object?
+#' @param debug logical; print debug messages?
+#' @export
+gdal_read_mdim = function(file, array_name = character(0), options = character(0), 
+						  offset = integer(0), count = integer(0), step = integer(0), 
+						  proxy = FALSE, debug = FALSE) {
+	CPL_read_mdim(file, array_name, options, offset, count, step, proxy, debug)
+}
+
+#' @name gdal
+#' @param dimx integer named vector with dimensions of object
+#' @param cdl list with variables, each having a named dim attribute
+#' @param wkt character; WKT of crs
+#' @param xy character; names of the spatial x and y dimension
+#' @param root_group_options character; driver specific options regarding the creation of the root group
+#' @param options character; driver specific options regarding reading or creating the dataset
+#' @param as_float logical; when \code{TRUE} write 4-byte floating point numbers, when \code{FALSE} write 8-byte doubles.
+#' @export
+gdal_write_mdim = function(file, driver, dimx, cdl, wkt, xy, ...,
+						   root_group_options = character(0), options = character(0),
+						   as_float = TRUE) {
+    CPL_write_mdim(file, driver, dimx, cdl, wkt, xy, root_group_options, options, as_float)
+}
+
+
+#' @name gdal
+#' @param f character; file name
+#' @param nxy integer vector of length 2
+#' @param values fill value
+#' @param crs object of class \code{crs}
+#' @param xlim numeric
+#' @param ylim numeric
+#' @export
+gdal_create = function(f, nxy, values, crs, xlim, ylim) {
+	CPL_create(as.character(f), as.integer(nxy), as.double(values), crs$wkt, as.double(xlim), as.double(ylim))
+}
+
+#' add or remove overviews to/from a raster image
+#'
+#' add or remove overviews to/from a raster image
+#' @param file character; file name
+#' @param overviews integer; overview levels
+#' @param method character; method to create overview; one of: nearest, average, rms, gauss, cubic, cubicspline, lanczos, average_mp, average_magphase, mode
+#' @param layers integer; layers to create overviews for (default: all)
+#' @param options character; dataset opening options
+#' @param clean logical; if \code{TRUE} only remove overviews, do not add
+#' @param read_only logical; if \code{TRUE}, add overviews to another file with extension \code{.ovr} added to \code{file}
+#' @return \code{TRUE}, invisibly, on success
+#' @seealso \link{gdal_utils} for access to other gdal utilities that have a C API
+#' @export
+gdal_addo = function(file, overviews = c(2,4,8,16), method = "NEAREST", layers = integer(0), 
+					 options = character(0), clean = FALSE, read_only = FALSE) {
+	stopifnot(length(method) == 1, is.character(method), is.numeric(overviews))
+	invisible(CPL_gdaladdo(file, method, as.integer(overviews), as.integer(layers), as.character(options), 
+				 as.logical(clean)[1], as.logical(read_only)[1]))
+}
