@@ -5,18 +5,16 @@
 
 # This is currently only used in `bind_rows()` and `bind_cols()`
 # because sf overrides all default implementations
+#' @name tidyverse
 dplyr_reconstruct.sf = function(data, template) {
 	sfc_name = attr(template, "sf_column")
 	if (inherits(template, "tbl_df"))
 		data = dplyr::as_tibble(data)
-
 	# Return a bare data frame is the geometry column is no longer there
 	if (!sfc_name %in% names(data))
 		return(data)
-
 	prec = st_precision(template)
 	crs = st_crs(template)
-
 	st_as_sf(
 		data,
 		sf_column_name = sfc_name,
@@ -38,9 +36,16 @@ group_split.sf <- function(.tbl, ..., .keep = TRUE) {
 #' @param ... other arguments
 #' @name tidyverse
 #' @examples
-#' library(dplyr)
-#' nc = st_read(system.file("shape/nc.shp", package="sf"))
-#' nc %>% filter(AREA > .1) %>% plot()
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc = read_sf(system.file("shape/nc.shp", package="sf"))
+#'  nc %>% filter(AREA > .1) %>% plot()
+#'  # plot 10 smallest counties in grey:
+#'  st_geometry(nc) %>% plot()
+#'  nc %>% select(AREA) %>% arrange(AREA) %>% slice(1:10) %>% plot(add = TRUE, col = 'grey')
+#'  title("the ten counties with smallest area")
+#'  nc2 <- nc %>% mutate(area10 = AREA/10)
+#'  nc %>% slice(1:2)
+#' }
 filter.sf <- function(.data, ..., .dots) {
 	agr = st_agr(.data)
 	class(.data) <- setdiff(class(.data), "sf")
@@ -50,9 +55,11 @@ filter.sf <- function(.data, ..., .dots) {
 #' @name tidyverse
 #' @examples
 #' # plot 10 smallest counties in grey:
-#' st_geometry(nc) %>% plot()
-#' nc %>% select(AREA) %>% arrange(AREA) %>% slice(1:10) %>% plot(add = TRUE, col = 'grey')
-#' title("the ten counties with smallest area")
+#' if (require(dplyr, quietly = TRUE)) {
+#'  st_geometry(nc) %>% plot()
+#'  nc %>% select(AREA) %>% arrange(AREA) %>% slice(1:10) %>% plot(add = TRUE, col = 'grey')
+#'  title("the ten counties with smallest area")
+#' }
 arrange.sf <- function(.data, ..., .dots) {
 	sf_column_name = attr(.data, "sf_column")
 	class(.data) = setdiff(class(.data), "sf")
@@ -62,8 +69,10 @@ arrange.sf <- function(.data, ..., .dots) {
 #' @name tidyverse
 #' @param add see corresponding function in dplyr
 #' @examples
-#' nc$area_cl = cut(nc$AREA, c(0, .1, .12, .15, .25))
-#' nc %>% group_by(area_cl) %>% class()
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc$area_cl = cut(nc$AREA, c(0, .1, .12, .15, .25))
+#'  nc %>% group_by(area_cl) %>% class()
+#' }
 group_by.sf <- function(.data, ..., add = FALSE) {
 	sf_column_name = attr(.data, "sf_column")
 	class(.data) <- setdiff(class(.data), "sf")
@@ -99,10 +108,11 @@ rowwise.sf <- function(x, ...) {
 		class = c("sf", class(x)))
 }
 
-
 #' @name tidyverse
 #' @examples
-#' nc2 <- nc %>% mutate(area10 = AREA/10)
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc2 <- nc %>% mutate(area10 = AREA/10)
+#' }
 mutate.sf <- function(.data, ..., .dots) {
 	#st_as_sf(NextMethod(), sf_column_name = attr(.data, "sf_column"))
 	agr = st_agr(.data)
@@ -111,10 +121,13 @@ mutate.sf <- function(.data, ..., .dots) {
 	.re_sf(NextMethod(), sf_column_name = sf_column_name, agr)
 }
 
+
 #' @name tidyverse
 #' @examples
-#' nc %>% transmute(AREA = AREA/10, geometry = geometry) %>% class()
-#' nc %>% transmute(AREA = AREA/10) %>% class()
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc %>% transmute(AREA = AREA/10, geometry = geometry) %>% class()
+#'  nc %>% transmute(AREA = AREA/10) %>% class()
+#' }
 transmute.sf <- function(.data, ..., .dots) {
 	sf_column_name = attr(.data, "sf_column")
 	agr = st_agr(.data)
@@ -125,10 +138,12 @@ transmute.sf <- function(.data, ..., .dots) {
 
 #' @name tidyverse
 #' @examples
-#' nc %>% select(SID74, SID79) %>% names()
-#' nc %>% select(SID74, SID79, geometry) %>% names()
-#' nc %>% select(SID74, SID79) %>% class()
-#' nc %>% select(SID74, SID79, geometry) %>% class()
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc %>% select(SID74, SID79) %>% names()
+#'  nc %>% select(SID74, SID79, geometry) %>% names()
+#'  nc %>% select(SID74, SID79) %>% class()
+#'  nc %>% select(SID74, SID79, geometry) %>% class()
+#' }
 #' @details \code{select} keeps the geometry regardless whether it is selected or not; to deselect it, first pipe through \code{as.data.frame} to let dplyr's own \code{select} drop it.
 select.sf <- function(.data, ...) {
 
@@ -168,7 +183,9 @@ select.sf <- function(.data, ...) {
 
 #' @name tidyverse
 #' @examples
-#' nc2 <- nc %>% rename(area = AREA)
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc2 <- nc %>% rename(area = AREA)
+#' }
 rename.sf <- function(.data, ...) {
 
 	if (!requireNamespace("tidyselect", quietly = TRUE))
@@ -212,13 +229,16 @@ rename.sf <- function(.data, ...) {
 
 #' @name tidyverse
 #' @examples
-#' nc %>% slice(1:2)
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc %>% slice(1:2)
+#' }
 slice.sf <- function(.data, ..., .dots) {
 	class(.data) <- setdiff(class(.data), "sf")
 	sf_column <- attr(.data, "sf_column")
 	st_as_sf(NextMethod(), sf_column_name = sf_column)
 }
 
+
 #' @name tidyverse
 #' @aliases summarise
 #' @param do_union logical; in case \code{summary} does not create a geometry column, should geometries be created by unioning using \link{st_union}, or simply by combining using \link{st_combine}? Using \link{st_union} resolves internal boundaries, but in case of unioning points, this will likely change the order of the points; see Details.
@@ -229,11 +249,13 @@ slice.sf <- function(.data, ..., .dots) {
 #'
 #' In case \code{do_union} is \code{FALSE}, \code{summarise} will simply combine geometries using \link{c.sfg}. When polygons sharing a boundary are combined, this leads to geometries that are invalid; see for instance \url{https://github.com/r-spatial/sf/issues/681}.
 #' @examples
-#' nc$area_cl = cut(nc$AREA, c(0, .1, .12, .15, .25))
-#' nc.g <- nc %>% group_by(area_cl)
-#' nc.g %>% summarise(mean(AREA))
-#' nc.g %>% summarise(mean(AREA)) %>% plot(col = grey(3:6 / 7))
-#' nc %>% as.data.frame %>% summarise(mean(AREA))
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc$area_cl = cut(nc$AREA, c(0, .1, .12, .15, .25))
+#'  nc.g <- nc %>% group_by(area_cl)
+#'  nc.g %>% summarise(mean(AREA))
+#'  nc.g %>% summarise(mean(AREA)) %>% plot(col = grey(3:6 / 7))
+#'  nc %>% as.data.frame %>% summarise(mean(AREA))
+#' }
 summarise.sf <- function(.data, ..., .dots, do_union = TRUE, is_coverage = FALSE) {
 	sf_column = attr(.data, "sf_column")
 	precision = st_precision(.data)
@@ -276,7 +298,9 @@ summarise.sf <- function(.data, ..., .dots, do_union = TRUE, is_coverage = FALSE
 #' @name tidyverse
 #' @param .keep_all see corresponding function in dplyr
 #' @examples
-#' nc[c(1:100, 1:10), ] %>% distinct() %>% nrow()
+#' if (require(dplyr, quietly = TRUE)) {
+#'  nc[c(1:100, 1:10), ] %>% distinct() %>% nrow()
+#' }
 #' @details \code{distinct} gives distinct records for which all attributes and geometries are distinct; \link{st_equals} is used to find out which geometries are distinct.
 distinct.sf <- function(.data, ..., .keep_all = FALSE) {
 	sf_column = attr(.data, "sf_column")
@@ -310,8 +334,9 @@ distinct.sf <- function(.data, ..., .keep_all = FALSE) {
 #' @param na.rm see original function docs
 #' @param factor_key see original function docs
 #' @examples
-#' library(tidyr)
-#' nc %>% select(SID74, SID79) %>% gather("VAR", "SID", -geometry) %>% summary()
+#' if (require(tidyr, quietly = TRUE) && require(dplyr, quietly = TRUE)) {
+#'  nc %>% select(SID74, SID79) %>% gather("VAR", "SID", -geometry) %>% summary()
+#' }
 gather.sf <- function(data, key, value, ..., na.rm = FALSE, convert = FALSE, factor_key = FALSE) {
 
 	if (! requireNamespace("rlang", quietly = TRUE))
@@ -329,6 +354,21 @@ gather.sf <- function(data, key, value, ..., na.rm = FALSE, convert = FALSE, fac
 		sf_column_name = attr(data, "sf_column"))
 }
 
+#' @name tidyverse
+#' @param template see original function docs
+#' @param data see original function docs
+#' @param cols see original function docs
+#' @param names_to see original function docs
+#' @param names_prefix see original function docs
+#' @param names_sep see original function docs
+#' @param names_pattern see original function docs
+#' @param names_ptypes see original function docs
+#' @param names_transform see original function docs
+#' @param names_repair see original function docs
+#' @param values_to see original function docs
+#' @param values_drop_na see original function docs
+#' @param values_ptypes see original function docs
+#' @param values_transform see original function docs
 pivot_longer.sf <- function (data, cols, names_to = "name", names_prefix = NULL,
 		names_sep = NULL, names_pattern = NULL, names_ptypes = NULL,
 		names_transform = NULL, names_repair = "check_unique",
@@ -363,16 +403,50 @@ pivot_longer.sf <- function (data, cols, names_to = "name", names_prefix = NULL,
   st_as_sf(out, sf_column_name = sf_column_name)
 }
 
+# https://github.com/r-spatial/sf/issues/1915
+#' @name tidyverse
+#' @export
+#' @param id_cols see original function docs
+#' @param names_from see original function docs
+#' @param names_prefix see original function docs
+#' @param names_sep see original function docs
+#' @param names_glue see original function docs
+#' @param names_sort see original function docs
+#' @param names_repair see original function docs
+#' @param values_from see original function docs
+#' @param values_fill see original function docs
+#' @param values_fn see original function docs
+pivot_wider.sf = function(data,
+                          id_cols = NULL,
+                          names_from, # = name,
+                          names_prefix = "",
+                          names_sep = "_",
+                          names_glue = NULL,
+                          names_sort = FALSE,
+                          names_repair = "check_unique",
+                          values_from, # = value,
+                          values_fill = NULL,
+                          values_fn = NULL,
+                          ...) {
+
+	agr = st_agr(data)
+	sf_column_name = attr(data, "sf_column")
+	class(data) = setdiff(class(data), "sf")
+	.re_sf(NextMethod(), sf_column_name = sf_column_name, agr)
+}
+
+
 
 #' @name tidyverse
 #' @param fill see original function docs
 #' @param drop see original function docs
 #' @examples
-#' library(tidyr)
-#' nc$row = 1:100 # needed for spread to work
-#' nc %>% select(SID74, SID79, geometry, row) %>%
+#' if (require(tidyr, quietly = TRUE) && require(dplyr, quietly = TRUE)) {
+#'  nc$row = 1:100 # needed for spread to work
+#'  nc %>% select(SID74, SID79, geometry, row) %>%
 #'		gather("VAR", "SID", -geometry, -row) %>%
 #'		spread(VAR, SID) %>% head()
+#' }
 spread.sf <- function(data, key, value, fill = NA, convert = FALSE, drop = TRUE,
 	        sep = NULL) {
 
@@ -403,12 +477,14 @@ sample_frac.sf <- function(tbl, size = 1, replace = FALSE, weight = NULL, .env =
 
 #' @name tidyverse
 #' @examples
-#' storms.sf = st_as_sf(storms, coords = c("long", "lat"), crs = 4326)
-#' x <- storms.sf %>% group_by(name, year) %>% nest
-#' trs = lapply(x$data, function(tr) st_cast(st_combine(tr), "LINESTRING")[[1]]) %>%
+#' if (require(tidyr, quietly = TRUE) && require(dplyr, quietly = TRUE)) {
+#'  storms.sf = st_as_sf(storms, coords = c("long", "lat"), crs = 4326)
+#'  x <- storms.sf %>% group_by(name, year) %>% nest
+#'  trs = lapply(x$data, function(tr) st_cast(st_combine(tr), "LINESTRING")[[1]]) %>%
 #'     st_sfc(crs = 4326)
-#' trs.sf = st_sf(x[,1:2], trs)
-#' plot(trs.sf["year"], axes = TRUE)
+#'  trs.sf = st_sf(x[,1:2], trs)
+#'  plot(trs.sf["year"], axes = TRUE)
+#' }
 #' @details \code{nest} assumes that a simple feature geometry list-column was among the columns that were nested.
 nest.sf = function (.data, ...) {
 
@@ -516,6 +592,13 @@ pillar_shaft.sfc <- function(x, ...) {
 	pillar::new_pillar_shaft_simple(out, align = "right", min_width = 25)
 }
 
+#' @name tidyverse
+drop_na.sf <- function(x, ...) {
+	sf_column_name = attr(x, "sf_column")
+	class(x) <- setdiff(class(x), "sf")
+	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
+}
+
 #nocov start
 register_all_s3_methods = function() {
 	has_dplyr_1.0 =
@@ -524,7 +607,6 @@ register_all_s3_methods = function() {
 
 	if (has_dplyr_1.0)
 		register_s3_method("dplyr", "dplyr_reconstruct", "sf")
-
 	register_s3_method("dplyr", "anti_join", "sf")
 	register_s3_method("dplyr", "arrange", "sf")
 	register_s3_method("dplyr", "distinct", "sf")
@@ -547,8 +629,10 @@ register_all_s3_methods = function() {
 	register_s3_method("dplyr", "summarise", "sf")
 	register_s3_method("dplyr", "transmute", "sf")
 	register_s3_method("dplyr", "ungroup", "sf")
+	register_s3_method("tidyr", "drop_na", "sf")
 	register_s3_method("tidyr", "gather", "sf")
 	register_s3_method("tidyr", "pivot_longer", "sf")
+	register_s3_method("tidyr", "pivot_wider", "sf")
 	register_s3_method("tidyr", "spread", "sf")
 	register_s3_method("tidyr", "nest", "sf")
 	register_s3_method("tidyr", "separate", "sf")
