@@ -185,8 +185,8 @@ Rcpp::List CPL_crs_parameters(Rcpp::List crs) {
 	if (srs == NULL)
 		Rcpp::stop("crs not found"); // #nocov
 
-	Rcpp::List out(13);
-	Rcpp::CharacterVector names(13);
+	Rcpp::List out(15);
+	Rcpp::CharacterVector names(15);
 	out(0) = Rcpp::NumericVector::create(srs->GetSemiMajor());
 	names(0) = "SemiMajor";
 
@@ -268,6 +268,31 @@ Rcpp::List CPL_crs_parameters(Rcpp::List crs) {
 #endif
 	names(12) = "ProjJson";
 
+	// WKT1_ESRI
+#if GDAL_VERSION_MAJOR >= 3
+	const char *options[3] = { "MULTILINE=YES", "FORMAT=WKT1_ESRI", NULL };
+	if (srs->exportToWkt(&cp, options) != OGRERR_NONE)
+		out(13) = Rcpp::CharacterVector::create(NA_STRING); // FIXME: CPLFree() in this case?
+	else {
+		out(13) = Rcpp::CharacterVector::create(cp);
+		CPLFree(cp);
+	}
+#else
+	out(13) = "";
+#endif
+	names(13) = "WKT1_ESRI";
+
+	// srid
+	if (srs->GetAuthorityName(NULL) != NULL && srs->GetAuthorityCode(NULL) != NULL) {
+		char str[101];
+		snprintf(str, (size_t) 100, "%s:%s",
+			srs->GetAuthorityName(NULL), srs->GetAuthorityCode(NULL));
+		Rcpp::CharacterVector v = str;
+		out(14) = v;
+	} else
+		out(14) = Rcpp::CharacterVector::create(NA_STRING);
+	names(14) = "srid";
+
 	set_error_handler();
 
 	delete srs;
@@ -505,31 +530,33 @@ Rcpp::List CPL_curve_to_linestring(Rcpp::List sfc) { // need to pass more parame
 
 // [[Rcpp::export]]
 Rcpp::List CPL_transform(Rcpp::List sfc, Rcpp::List crs,
-		Rcpp::NumericVector AOI, Rcpp::CharacterVector pipeline, bool reverse = false) {
-
-	// import crs:
-	OGRSpatialReference *dest = OGRSrs_from_crs(crs);
-	if (dest == NULL)
-		Rcpp::stop("crs not found: is it missing?"); // #nocov
+		Rcpp::NumericVector AOI, Rcpp::CharacterVector pipeline, bool reverse = false,
+		double desired_accuracy = -1.0, bool allow_ballpark = true) {
 
 	// transform geometries:
 	std::vector<OGRGeometry *> g = ogr_from_sfc(sfc, NULL);
-	if (g.size() == 0) {
-		dest->Release();
+	if (g.size() == 0)
 		return sfc_from_ogr(g, true); // destroys g
-	}
+
+	OGRSpatialReference *dest = NULL;
+	// if pipeline was not set, import crs to dest:
+	if (pipeline.size() == 0 && !(dest = OGRSrs_from_crs(crs)))
+		Rcpp::stop("crs not found: is it missing?"); // #nocov
+
 #if GDAL_VERSION_MAJOR >= 3
 	OGRCoordinateTransformationOptions *options = new OGRCoordinateTransformationOptions;
-	if (pipeline.size() || AOI.size()) {
-		if (AOI.size() && !options->SetAreaOfInterest(AOI[0], AOI[1], AOI[2], AOI[3])) // and pray!
-			Rcpp::stop("values for area of interest not accepted");
-		if (pipeline.size() && !options->SetCoordinateOperation(pipeline[0], reverse))
-			Rcpp::stop("pipeline value not accepted");
-	}
-	unset_error_handler(); // FIXME: is this always a good idea?
+	if (pipeline.size() && !options->SetCoordinateOperation(pipeline[0], reverse))
+		Rcpp::stop("pipeline value not accepted");
+	if (AOI.size() == 4 && !options->SetAreaOfInterest(AOI[0], AOI[1], AOI[2], AOI[3]))
+		Rcpp::stop("values for area of interest not accepted");
+#if GDAL_VERSION_MINOR >= 3
+	options->SetDesiredAccuracy(desired_accuracy);
+	options->SetBallparkAllowed(allow_ballpark);
+#endif
+	// unset_error_handler(); // FIXME: is this always a good idea?
 	OGRCoordinateTransformation *ct =
 		OGRCreateCoordinateTransformation(g[0]->getSpatialReference(), dest, *options);
-	set_error_handler();
+	// set_error_handler();
 	delete options;
 #else
 	if (pipeline.size() || AOI.size())
@@ -538,9 +565,10 @@ Rcpp::List CPL_transform(Rcpp::List sfc, Rcpp::List crs,
 		OGRCreateCoordinateTransformation(g[0]->getSpatialReference(), dest);
 #endif
 	if (ct == NULL) {
-		dest->Release(); // #nocov start
+		if (dest)
+			dest->Release(); // #nocov start
 		sfc_from_ogr(g, true); // to destroy g
-		Rcpp::stop("OGRCreateCoordinateTransformation() returned NULL: PROJ available?"); // #nocov end
+		Rcpp::stop("OGRCreateCoordinateTransformation(): transformation not available"); // #nocov end
 	}
 	for (size_t i = 0; i < g.size(); i++) {
 		CPLPushErrorHandler(CPLQuietErrorHandler);
@@ -557,8 +585,11 @@ Rcpp::List CPL_transform(Rcpp::List sfc, Rcpp::List crs,
 	}
 
 	Rcpp::List ret = sfc_from_ogr(g, true); // destroys g;
+	// how to return the target CRS when only a transformation pipeline is provided? Not by:
+	// ret.attr("crs") = create_crs(ct->GetTargetCS(), true);
 	ct->DestroyCT(ct);
-	dest->Release();
+	if (dest)
+		dest->Release();
 	return ret;
 }
 
@@ -576,7 +607,7 @@ Rcpp::List CPL_wrap_dateline(Rcpp::List sfc, Rcpp::CharacterVector opt, bool qui
 }
 
 // [[Rcpp::export]]
-Rcpp::List CPL_get_rgdal_drivers(int dummy) {
+Rcpp::List CPL_get_gdal_drivers(int dummy) {
 
 	int ndr = GetGDALDriverManager()->GetDriverCount();
 	Rcpp::CharacterVector name(ndr);
