@@ -153,9 +153,11 @@ Rcpp::List CPL_get_layers(Rcpp::CharacterVector datasource, Rcpp::CharacterVecto
 	Rcpp::List geomtype(poDS->GetLayerCount());
 	Rcpp::NumericVector field_count(poDS->GetLayerCount());
 	Rcpp::NumericVector feature_count(poDS->GetLayerCount());
+	Rcpp::List layer_crs(poDS->GetLayerCount());
 
 	for(int iLayer = 0; iLayer < poDS->GetLayerCount(); iLayer++) {
 		OGRLayer *poLayer = poDS->GetLayer(iLayer);
+		layer_crs[iLayer] = create_crs(poLayer->GetSpatialRef());
 		names(iLayer) = poLayer->GetName();
 		int nGeomFieldCount = poLayer->GetLayerDefn()->GetGeomFieldCount();
 		if (nGeomFieldCount == 0) {
@@ -180,14 +182,15 @@ Rcpp::List CPL_get_layers(Rcpp::CharacterVector datasource, Rcpp::CharacterVecto
 			feature_count(iLayer) = count_features(poLayer);
 	}
 
-	Rcpp::List out(5);
+	Rcpp::List out(6);
 	out(0) = names;
 	out(1) = geomtype;
 	out(2) = poDS->GetDriverName();
 	out(3) = feature_count;
 	out(4) = field_count;
+	out(5) = layer_crs;
 	GDALClose(poDS); // close & destroys data source
-	out.attr("names") = Rcpp::CharacterVector::create("name", "geomtype", "driver", "features", "fields");
+	out.attr("names") = Rcpp::CharacterVector::create("name", "geomtype", "driver", "features", "fields", "crs");
 	out.attr("class") = Rcpp::CharacterVector::create("sf_layers");
 	return out;
 }
@@ -277,16 +280,28 @@ Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
 				case OFTDate: {
 					int Year, Month, Day, Hour, Minute, TZFlag;
 					float Second;
+					const char *tzone = "";
 					poFeature->GetFieldAsDateTime(iField, &Year, &Month, &Day, &Hour, &Minute,
 						&Second, &TZFlag);
+					if (TZFlag == 100)
+						tzone = "UTC";
 					//  POSIXlt: sec   min  hour  mday   mon  year  wday  yday isdst ...
 					Rcpp::List dtlst =
-						Rcpp::List::create((double) Second, (double) Minute,
-						(double) Hour, (double) Day, (double) Month - 1, (double) Year - 1900,
-						0.0, 0.0, 0.0);
-					dtlst.attr("class") = "POSIXlt";
+						Rcpp::List::create(
+							Rcpp::_["sec"] = (double) Second, 
+							Rcpp::_["min"] = (int) Minute,
+							Rcpp::_["hour"] = (int) Hour,
+							Rcpp::_["mday"] = (int) Day,
+							Rcpp::_["mon"] = (int) Month - 1,
+							Rcpp::_["year"] = (int) Year - 1900,
+							Rcpp::_["wday"] = NA_INTEGER, 
+							Rcpp::_["yday"] = NA_INTEGER, 
+							Rcpp::_["isdst"] = NA_INTEGER,
+							Rcpp::_["zone"] = tzone,
+							Rcpp::_["gmtoff"] = NA_INTEGER);
 					if (TZFlag == 100)
 						dtlst.attr("tzone") = "UTC";
+					dtlst.attr("class") = "POSIXlt";
 					Rcpp::NumericVector nv;
 					nv = out[iField];
 					if (! not_NA) {
@@ -576,7 +591,7 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 
 	if (! quiet) {
 		if (! Rcpp::CharacterVector::is_na(query[0]))
-			Rcpp::Rcout << "Reading query `" << query[0] << "' from data source ";
+			Rcpp::Rcout << "Reading query `" << query[0] << "'" << std::endl << "from data source ";
 		else
 			Rcpp::Rcout << "Reading layer `" << layer[0] << "' from data source ";
 		// if (LENGTH(datasource[0]) > (width - (34 + LENGTH(layer[0]))))
