@@ -52,17 +52,56 @@ Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVec
 	std::vector <char *> options_char = create_options(options, true);
 	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALInfoOptions* opt = GDALInfoOptionsNew(options_char.data(), NULL);
-	GDALDatasetH ds = GDALOpenEx((const char *) obj[0], GA_ReadOnly, NULL, oo_char.data(), NULL);
-	if (ds == NULL)
-		return 1; // #nocov
+	GDALDatasetH ds = NULL;
+	if (obj.size())
+		ds = GDALOpenEx((const char *) obj[0], GA_ReadOnly, NULL, oo_char.data(), NULL);
 	char *ret_val = GDALInfo(ds, opt);
+	if (ret_val == NULL)
+		return Rcpp::CharacterVector::create();
 	Rcpp::CharacterVector ret = ret_val; // copies
 	CPLFree(ret_val);
 	GDALInfoOptionsFree(opt);
-	GDALClose(ds);
+	if (ds)
+		GDALClose(ds);
 	return ret;
 }
 
+// [[Rcpp::export]]
+Rcpp::LogicalVector CPL_gdaladdo(Rcpp::CharacterVector obj, Rcpp::CharacterVector method, 
+		Rcpp::IntegerVector overviews, Rcpp::IntegerVector bands, Rcpp::CharacterVector oo,
+		bool clean = false, bool read_only = false) {
+
+	/*
+	std::vector <char *> options_char = create_options(options, true);
+	GDALInfoOptions* opt = GDALInfoOptionsNew(options_char.data(), NULL);
+	*/
+	std::vector <char *> oo_char = create_options(oo, true); // open options
+
+	GDALDatasetH ds;
+	if ((ds = GDALOpenEx((const char *) obj[0], 
+						GDAL_OF_RASTER | (read_only ? GA_ReadOnly : GA_Update),
+						NULL, oo_char.data(), NULL)) == NULL)
+		Rcpp::stop(read_only ? "cannot open file for reading" : "cannot open file for writing");
+
+	if (clean) { // remove overviews:
+		if (GDALBuildOverviews(ds, method[0], 0, nullptr,
+                               0, nullptr, GDALRProgress, nullptr) != CE_None) {
+			GDALClose(ds);
+			Rcpp::stop("error while cleaning overviews");
+		}
+	} else { // build overviews:
+		if (GDALBuildOverviews(ds, method[0], 
+					overviews.size(), overviews.size() ? &(overviews[0]) : NULL,
+					bands.size(), bands.size() ? &(bands[0]) : NULL, 
+					GDALRProgress, NULL) != CE_None) {
+			GDALClose(ds);
+			Rcpp::stop("error while building overviews");
+		}
+	}
+	GDALClose(ds);
+	return true;
+}
+
 // #nocov start
 
 // [[Rcpp::export]]
@@ -328,21 +367,23 @@ Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 Rcpp::CharacterVector CPL_gdalmdiminfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options,
 		Rcpp::CharacterVector oo) { 
 	std::vector <char *> oo_char = create_options(oo, true);
-	GDALDatasetH ds = GDALOpenEx((const char *) obj[0], GA_ReadOnly, NULL, oo_char.data(), NULL);
-	if (ds == NULL)
-		return 1; // #nocov
+	GDALDatasetH ds = GDALOpenEx((const char *) obj[0], GDAL_OF_MULTIDIM_RASTER | GDAL_OF_VERBOSE_ERROR , NULL, oo_char.data(), NULL);
+	if (ds == NULL) {
+		Rcpp::Rcout << "failed to open " << obj[0] << std::endl;
+		Rcpp::stop("Error opening data source");
+	}
 	std::vector <char *> options_char = create_options(options, true);
 	GDALMultiDimInfoOptions* opt = GDALMultiDimInfoOptionsNew(options_char.data(), NULL);
 	char *ret_val = GDALMultiDimInfo(ds, opt);
 	GDALMultiDimInfoOptionsFree(opt);
 	GDALClose(ds);
-	Rcpp::CharacterVector ret(1);
 	if (ret_val != NULL) {
-		ret[0] = ret_val; // copies?
+		Rcpp::CharacterVector ret(1);
+		ret[0] = ret_val;
 		CPLFree(ret_val);
+		return ret;
 	} else
 		Rcpp::stop("GDALMultiDimInfo returned NULL");
-	return ret;
 }
 
 // [[Rcpp::export]]
@@ -358,21 +399,21 @@ Rcpp::LogicalVector CPL_gdalmdimtranslate(Rcpp::CharacterVector src, Rcpp::Chara
 
 	if (! quiet)
 		GDALMultiDimTranslateOptionsSetProgress(opt, GDALRProgress, NULL);
-	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, 
-		NULL, oo_char.data(), NULL);
-	if (src_pt == NULL)
-		return 1; // #nocov
 	std::vector<GDALDatasetH> srcpt(src.size());
 	for (int i = 0; i < src.size(); i++) {
-		srcpt[i] = GDALOpenEx((const char *) src[i], GDAL_OF_RASTER | GA_ReadOnly, NULL, 
+		srcpt[i] = GDALOpenEx((const char *) src[i], GDAL_OF_RASTER | GDAL_OF_MULTIDIM_RASTER | GDAL_OF_VERBOSE_ERROR, NULL, 
 			oo_char.data(), NULL);
-		if (srcpt[i] == NULL)
-			Rcpp::stop("cannot open source dataset");
+		if (srcpt[i] == NULL) {
+			Rcpp::Rcout << "dataset: " << src[i] << ": " << std::endl;
+			Rcpp::stop("Cannot open source dataset");
+		}
 	}
 	GDALDatasetH result = GDALMultiDimTranslate((const char *) dst[0], NULL, srcpt.size(), srcpt.data(), opt, &err);
 	GDALMultiDimTranslateOptionsFree(opt);
 	if (result != NULL)
 		GDALClose(result);
+	else 
+		Rcpp::stop("failed to open destination data set");
 	for (int i = 0; i < src.size(); i++)
 		GDALClose(srcpt[i]);
 	return result == NULL || err;
