@@ -2,12 +2,12 @@
 #include <geos_c.h>
 
 #if GEOS_VERSION_MAJOR == 3
+# if GEOS_VERSION_MINOR >= 4
+#  define HAVE340
+# endif
 # if GEOS_VERSION_MINOR >= 5
 #  define HAVE350
 # endif
-# if GEOS_VERSION_MINOR >= 8
-#  define HAVE380
-# endif
 # if GEOS_VERSION_MINOR == 6
 #  if GEOS_VERSION_PATCH >= 1
 #   define HAVE361
@@ -17,12 +17,29 @@
 #  define HAVE361
 #  define HAVE370
 # endif
+# if GEOS_VERSION_MINOR >= 8
+#  define HAVE380
+# endif
+# if GEOS_VERSION_MINOR >= 9
+#  define HAVE390
+# endif
+# if GEOS_VERSION_MINOR == 10
+#  if GEOS_VERSION_PATCH >= 1
+#   define HAVE3101
+#  endif
+# endif
+# if GEOS_VERSION_MINOR >= 10
+#  define HAVE3101
+# endif
 #else
 # if GEOS_VERSION_MAJOR > 3
+#  define HAVE340
 #  define HAVE350
 #  define HAVE370
 #  define HAVE361
 #  define HAVE380
+#  define HAVE390
+#  define HAVE3101
 # endif
 #endif
 
@@ -40,6 +57,8 @@ typedef char (* log_prfn)(GEOSContextHandle_t, const GEOSPreparedGeometry *,
 	const GEOSGeometry *);
 typedef GEOSGeom (* geom_fn)(GEOSContextHandle_t, const GEOSGeom, const GEOSGeom);
 
+static int notice = 0; // global var to silently catch notice of illegal geoms, e.g. non-closed rings
+
 void cb(void *item, void *userdata) { // callback function for tree selection
 	std::vector<size_t> *ret = (std::vector<size_t> *) userdata;
 	ret->push_back(*((size_t *) item));
@@ -50,7 +69,7 @@ static void __errorHandler(const char *fmt, ...) { // #nocov start
 	char buf[BUFSIZ], *p;
 	va_list ap;
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, (size_t) BUFSIZ, fmt, ap);
 	va_end(ap);
 	p = buf + strlen(buf) - 1;
 	if(strlen(buf) > 0 && *p == '\n') *p = '\0';
@@ -66,7 +85,7 @@ static void __warningHandler(const char *fmt, ...) {
 	char buf[BUFSIZ], *p;
 	va_list ap;
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, (size_t) BUFSIZ, fmt, ap);
 	va_end(ap);
 	p = buf + strlen(buf) - 1;
 	if(strlen(buf) > 0 && *p == '\n') *p = '\0';
@@ -131,7 +150,7 @@ static std::vector<GEOSGeometry*> to_raw(std::vector<GeomPtr> & g) {
 	return raw;
 }
 
-std::vector<GeomPtr> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::List sfc, int *dim = NULL) {
+std::vector<GeomPtr> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::List sfc, int *dim = NULL, bool stop_on_NULL = true) {
 
 	Rcpp::List sfc_cls = get_dim_sfc(sfc);
 	Rcpp::CharacterVector cls = sfc_cls["_cls"];
@@ -145,12 +164,29 @@ std::vector<GeomPtr> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::Li
 	if (cls[0] == "XYM" || cls[0] == "XYZM")
 		Rcpp::stop("GEOS does not support XYM or XYZM geometries; use st_zm() to drop M\n"); // #nocov
 
+#ifdef HAVE_390
+	double precision = sfc.attr("precision");
+	bool set_precision = precision != 0.0;
+	if (set_precision)
+		precision = 1/precision;
+	sfc.attr("precision") = 0.0; // so that CPL_write_wkb doesn't do the rounding;
+#endif
 	Rcpp::List wkblst = CPL_write_wkb(sfc, true);
 	std::vector<GeomPtr> g(sfc.size());
 	GEOSWKBReader *wkb_reader = GEOSWKBReader_create_r(hGEOSCtxt);
 	for (int i = 0; i < sfc.size(); i++) {
 		Rcpp::RawVector r = wkblst[i];
 		g[i] = geos_ptr(GEOSWKBReader_read_r(hGEOSCtxt, wkb_reader, &(r[0]), r.size()), hGEOSCtxt);
+		if (g[i].get() == NULL) {
+			if (stop_on_NULL) {
+				Rcpp::Rcout << "While converting geometry of record: " << i << " to GEOS:" << std::endl;
+				Rcpp::stop("Illegal geometry found: fix manually, or filter out using st_is_valid() and is.na()\n");
+			}
+		}
+#ifdef HAVE_390
+		else if (set_precision)
+			g[i] = geos_ptr(GEOSGeom_setPrecision_r(hGEOSCtxt, g[i].get(), precision, 0), hGEOSCtxt);
+#endif
 	}
 	GEOSWKBReader_destroy_r(hGEOSCtxt, wkb_reader);
 	return g;
@@ -480,15 +516,19 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 Rcpp::CharacterVector CPL_geos_is_valid_reason(Rcpp::List sfc) {
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 
-	std::vector<GeomPtr> gmv = geometries_from_sfc(hGEOSCtxt, sfc, NULL);
+	std::vector<GeomPtr> gmv = geometries_from_sfc(hGEOSCtxt, sfc, NULL, false);
 	Rcpp::CharacterVector out(gmv.size());
 	for (int i = 0; i < out.length(); i++) {
-		char *buf = GEOSisValidReason_r(hGEOSCtxt, gmv[i].get());
-		if (buf == NULL)
-			out[i] = NA_STRING; // #nocov
+		if (gmv[i].get() == NULL)
+			out[i] = NA_STRING;
 		else {
-			out[i] = buf;
-			GEOSFree_r(hGEOSCtxt, buf);
+			char *buf = GEOSisValidReason_r(hGEOSCtxt, gmv[i].get());
+			if (buf == NULL)
+				out[i] = NA_STRING; // #nocov
+			else {
+				out[i] = buf;
+				GEOSFree_r(hGEOSCtxt, buf);
+			}
 		}
 	}
 	CPL_geos_finish(hGEOSCtxt);
@@ -497,14 +537,28 @@ Rcpp::CharacterVector CPL_geos_is_valid_reason(Rcpp::List sfc) {
 
 // #nocov start - no GEOS 3.8.0 on travis yet
 // [[Rcpp::export]]
-Rcpp::List CPL_geos_make_valid(Rcpp::List sfc) {
+Rcpp::List CPL_geos_make_valid(Rcpp::List sfc, std::string method, bool keep_collapsed) {
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 
 	std::vector<GeomPtr> gmv = geometries_from_sfc(hGEOSCtxt, sfc, NULL);
 	std::vector<GeomPtr> out(gmv.size());
 #ifdef HAVE380
+# ifdef HAVE3101
+	GEOSMakeValidParams *makeValidParams = GEOSMakeValidParams_create_r(hGEOSCtxt);
+	if (method == "valid_linework")
+		GEOSMakeValidParams_setMethod_r(hGEOSCtxt, makeValidParams, GEOS_MAKE_VALID_LINEWORK);
+	else if (method == "valid_structure")
+		GEOSMakeValidParams_setMethod_r(hGEOSCtxt, makeValidParams, GEOS_MAKE_VALID_STRUCTURE);
+	else
+		Rcpp::stop("geos_method not recognized");
+	GEOSMakeValidParams_setKeepCollapsed_r(hGEOSCtxt, makeValidParams, (int) keep_collapsed);
+	for (size_t i = 0; i < gmv.size(); i++)
+		gmv[i] = geos_ptr(GEOSMakeValidWithParams_r(hGEOSCtxt, gmv[i].get(), makeValidParams), hGEOSCtxt);
+	GEOSMakeValidParams_destroy_r(hGEOSCtxt, makeValidParams);
+# else
 	for (size_t i = 0; i < gmv.size(); i++)
 		gmv[i] = geos_ptr(GEOSMakeValid_r(hGEOSCtxt, gmv[i].get()), hGEOSCtxt);
+# endif
 #else
 	Rcpp::stop("this shouldn't happen: st_make_valid should use lwgeom");
 #endif
@@ -518,10 +572,12 @@ Rcpp::List CPL_geos_make_valid(Rcpp::List sfc) {
 Rcpp::LogicalVector CPL_geos_is_valid(Rcpp::List sfc, bool NA_on_exception = true) {
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 
-	int notice = 0;
+	notice = 0;
 	if (NA_on_exception) {
+/*
 		if (sfc.size() > 1)
 			Rcpp::stop("NA_on_exception will only work reliably with length 1 sfc objects"); // #nocov
+*/
 #ifdef HAVE350
 		GEOSContext_setNoticeMessageHandler_r(hGEOSCtxt,
 			(GEOSMessageHandler_r) __emptyNoticeHandler, (void *) &notice);
@@ -529,14 +585,29 @@ Rcpp::LogicalVector CPL_geos_is_valid(Rcpp::List sfc, bool NA_on_exception = tru
 			(GEOSMessageHandler_r) __countErrorHandler, (void *) &notice);
 #endif
 	}
-	std::vector<GeomPtr> gmv = geometries_from_sfc(hGEOSCtxt, sfc, NULL); // where notice might be set!
-	Rcpp::LogicalVector out(gmv.size());
+	Rcpp::LogicalVector out(sfc.size());
 	for (int i = 0; i < out.length(); i++) {
-		int ret = GEOSisValid_r(hGEOSCtxt, gmv[i].get());
+		// get geometry i:
+		Rcpp::List geom_i = Rcpp::List::create(sfc[i]);
+		geom_i.attr("precision") = sfc.attr("precision");
+		geom_i.attr("class") = sfc.attr("class");
+		geom_i.attr("crs") = sfc.attr("crs");
+		SEXP classes = sfc.attr("classes");
+		if (classes != R_NilValue) {
+			Rcpp::CharacterVector cl = sfc.attr("classes");
+			geom_i.attr("classes") = cl[i];
+		}
+		std::vector<GeomPtr> gmv = geometries_from_sfc(hGEOSCtxt, geom_i, NULL, false); // where notice might be set!
+		int ret;
+		if (gmv[0].get() == NULL)
+			ret = 2;
+		else
+			ret = GEOSisValid_r(hGEOSCtxt, gmv[0].get());
 		if (NA_on_exception && (ret == 2 || notice != 0))
 			out[i] = NA_LOGICAL; // no need to set notice back here, as we only consider 1 geometry #nocov
 		else
 			out[i] = chk_(ret);
+		notice = 0; // reset notice.
 	}
 #ifdef HAVE350
 	GEOSContext_setNoticeHandler_r(hGEOSCtxt, __warningHandler);
@@ -717,7 +788,8 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(
 					preserveTopology[i] ?
-						chkNULL(GEOSTopologyPreserveSimplify_r(hGEOSCtxt, g[i].get(), dTolerance[i])) :
+						chkNULL(GEOSTopologyPreserveSimplify_r(hGEOSCtxt, g[i].get(),
+							dTolerance[i])) :
 						chkNULL(GEOSSimplify_r(hGEOSCtxt, g[i].get(), dTolerance[i])), hGEOSCtxt);
 	} else if (op == "linemerge") {
 		for (size_t i = 0; i < g.size(); i++)
@@ -732,13 +804,6 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 			out[i] = geos_ptr(chkNULL(GEOSGetCentroid_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
 		}
 	} else
-#ifdef HAVE370
-	if (op == "reverse") {
-		for (size_t i = 0; i < g.size(); i++) {
-			out[i] = geos_ptr(chkNULL(GEOSReverse_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
-		}
-	} else
-#endif
 	if (op == "node") {
 		for (size_t i = 0; i < g.size(); i++) {
 			out[i] = geos_ptr(chkNULL(GEOSNode_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
@@ -748,10 +813,30 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 			out[i] = geos_ptr(chkNULL(GEOSPointOnSurface_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
 		}
 	} else
-#if GEOS_VERSION_MAJOR >= 3 && GEOS_VERSION_MINOR >= 4
+#ifdef HAVE340
 	if (op == "triangulate") {
 		for (size_t i = 0; i < g.size(); i++)
-			out[i] = geos_ptr(chkNULL(GEOSDelaunayTriangulation_r(hGEOSCtxt, g[i].get(), dTolerance[i], bOnlyEdges)), hGEOSCtxt);
+			out[i] = geos_ptr(chkNULL(GEOSDelaunayTriangulation_r(hGEOSCtxt, g[i].get(),
+				dTolerance[i], bOnlyEdges)), hGEOSCtxt);
+	} else
+#endif
+#ifdef HAVE370
+	if (op == "reverse") {
+		for (size_t i = 0; i < g.size(); i++) {
+			out[i] = geos_ptr(chkNULL(GEOSReverse_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
+		}
+	} else
+#endif
+#ifdef HAVE390
+	if (op == "inscribed_circle") {
+		for (size_t i = 0; i < g.size(); i++) {
+			out[i] = geos_ptr(chkNULL(GEOSMaximumInscribedCircle_r(hGEOSCtxt, g[i].get(),
+				dTolerance[i])), hGEOSCtxt);
+		}
+	} else if (op == "minimum_rotated_rectangle") {
+		for (size_t i = 0; i < g.size(); i++) {
+			out[i] = geos_ptr(chkNULL(GEOSMinimumRotatedRectangle_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
+		}
 	} else
 #endif
 		Rcpp::stop("invalid operation"); // #nocov
@@ -1097,7 +1182,7 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 	std::vector<GeomPtr> out;
 	int errors = 0;
 #ifdef HAVE350
-	int notice = 0;
+	notice = 0;
 	GEOSContext_setNoticeMessageHandler_r(hGEOSCtxt,
 		(GEOSMessageHandler_r) __emptyNoticeHandler, (void *) &notice);
 	GEOSContext_setErrorMessageHandler_r(hGEOSCtxt,
@@ -1124,13 +1209,17 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 				// iterate over items in query and erase overlapping areas in geom
 				for (size_t j = 0; j < tree_sel.size(); j++) {
 					size_t k = tree_sel[j];
-					GeomPtr inters = nullptr;
+					GeomPtr inters = geos_ptr(GEOSIntersection_r(hGEOSCtxt, out[k].get(), geom.get()), hGEOSCtxt);
 					if (geom.get() != nullptr) {
-						inters = geos_ptr(GEOSIntersection_r(hGEOSCtxt, out[k].get(), geom.get()), hGEOSCtxt);
 						if (inters == nullptr)
 							errors++;
-						if (inters != nullptr && !chk_(GEOSisEmpty_r(hGEOSCtxt, inters.get()))) { // i and k intersection
-							GeomPtr g = geos_ptr(GEOSDifference_r(hGEOSCtxt, out[k].get(), inters.get()), hGEOSCtxt); // cut out inters from out[k]
+						else if (!chk_(GEOSisEmpty_r(hGEOSCtxt, inters.get()))) { // i and k intersection
+							// cut out inters from geom:
+							geom = geos_ptr(GEOSDifference_r(hGEOSCtxt, geom.get(), inters.get()), hGEOSCtxt); 
+							if (geom == nullptr)
+								Rcpp::stop("GEOS exception"); // #nocov
+							// cut out inters from out[k]:
+							GeomPtr g = geos_ptr(GEOSDifference_r(hGEOSCtxt, out[k].get(), inters.get()), hGEOSCtxt); 
 							if (g == nullptr)
 								Rcpp::warning("GEOS difference returns NULL"); // #nocov
 							else {
@@ -1160,6 +1249,7 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 #ifdef HAVE350
 	if (notice > 0)
 		Rcpp::warning("one or more notices ignored");
+	notice = 0;
 	GEOSContext_setNoticeHandler_r(hGEOSCtxt, __warningHandler);
 	GEOSContext_setErrorHandler_r(hGEOSCtxt, __errorHandler);
 #endif
