@@ -105,6 +105,12 @@ Rcpp::List CPL_polygonize(Rcpp::CharacterVector raster, Rcpp::CharacterVector ma
 		OGRFieldDefn oField("Value", OFTReal);
 		if (poLayer->CreateField(&oField) != OGRERR_NONE)
     		Rcpp::stop("Creating attribute field failed.\n"); // #nocov
+		OGRFieldDefn minField("Min", OFTReal);
+		if (poLayer->CreateField(&minField) != OGRERR_NONE)
+    		Rcpp::stop("Creating attribute field failed.\n"); // #nocov
+		OGRFieldDefn maxField("Max", OFTReal);
+		if (poLayer->CreateField(&maxField) != OGRERR_NONE)
+    		Rcpp::stop("Creating attribute field failed.\n"); // #nocov
 
 		if (!use_contours) {
 			if (GDALFPolygonize((GDALRasterBandH) poBand, maskBand,
@@ -153,10 +159,14 @@ Rcpp::List CPL_rasterize(Rcpp::CharacterVector raster, Rcpp::CharacterVector ras
 	
 	std::vector<OGRGeometry *> geoms = ogr_from_sfc(sfc, NULL);
 
-	int bandlist = 1;
+	// int bandlist = 1;
+	std::vector<int> bandlist(poDataset->GetRasterCount());
+	for (size_t i = 0; i < bandlist.size(); i++)
+		bandlist[i] = (int) i+1; // 1-based
+
 	CPLErr err = GDALRasterizeGeometries((GDALDatasetH) poDataset, // hDS,
-		1, // int 	nBandCount,
-		&bandlist, // int * 	panBandList,
+		poDataset->GetRasterCount(), // int 	nBandCount,
+		bandlist.data(), // int * 	panBandList,
 		geoms.size(), // int 	nGeomCount,
 		(OGRGeometryH *) geoms.data(), // OGRGeometryH * 	pahGeometries,
 		NULL, // GDALTransformerFunc 	pfnTransformer,
